/** This class represents a feature vector storing the secondary structure features for a RNA  *  sequence.
 *
 * @author Sumeet Agarwal
 * @version 20.7.2005.
 */
public class FV
{
	public String seq;
	public double prob;
	public String path;
	public int length;
	public float[] comp = new float[4];
	public int loop;
	public int aBulges;
	public int sBulges;
	public int substems;
	public float avgABulgeLen;
	public float avgSBulgeLen;
	public float avgSubstemLen;
	public int maxABulgeLen;
	public int maxSBulgeLen;
	public int maxSubstemLen;
	public int minABulgeLen;
	public int minSBulgeLen;
	public int minSubstemLen;

	/* Constructor; does the job of generating the actual feature values and storing them in 
         * the corresponding variables.
	 *
         * @param s The input RNA sequence
         * @param pr The likelihood of the given alignment with the csHMM
         * @param p The actual alignment, as a state sequence
         */
	public FV(String s, double pr, String p){
		seq = s;
		prob = Math.pow(pr, (double)1/s.length());   
		path = p;
		length = seq.length();
		for(int i=0; i<length; i++){
			if(seq.charAt(i) == 'a' || seq.charAt(i) == 'A')
				comp[0] += 1;
			else if(seq.charAt(i) == 'c' || seq.charAt(i) == 'C')
                     		           	comp[1] += 1;
			else if(seq.charAt(i) == 'g' || seq.charAt(i) == 'G')
                                			comp[2] += 1;
			else if(seq.charAt(i) == 'u' || seq.charAt(i) == 'U' || seq.charAt(i) == 't' || seq.charAt(i) == 'T')
                               			 comp[3] += 1;
		}
		for(int i=0; i<4; i++)
			comp[i] /= length;
		int curr = 0;	
		for(int i=0; i<length; i++){
			if(path.charAt(i) == '3')
				loop++;
			else if(path.charAt(i) == '2'){
				avgABulgeLen++;
				curr++;
 				if (i==0 || path.charAt(i-1) == '1')
					aBulges++;
				if(i==length-1 || path.charAt(i+1) != '2'){
					if(curr > maxABulgeLen)
						maxABulgeLen = curr;
					if(minABulgeLen == 0 || curr < minABulgeLen)
						minABulgeLen = curr;
					curr = 0;
				}
			}
			else if(path.charAt(i) == '4'){
				avgABulgeLen++;
				curr++;
				if(path.charAt(i-1) == '5')
					aBulges++;
				else if(path.charAt(i-1) == '6'){
					sBulges--;
					aBulges++;
					int j=i-1;
					while(path.charAt(j) == '6'){
						avgSBulgeLen--;
						avgABulgeLen++;
						j--;
					}
				}
				if(i==length-1 || path.charAt(i+1) != '4'){
					if(curr > maxABulgeLen)
						maxABulgeLen = curr;
					if(minABulgeLen == 0 || curr < minABulgeLen)
						minABulgeLen = curr;
					curr = 0;
				}
			}
			else if(path.charAt(i) == '5'){
				avgSubstemLen++;
				curr++;
				if(path.charAt(i-1) != '5')
					substems++;
				if(i==length-1 || path.charAt(i+1) != '5'){
					if(curr > maxSubstemLen)
						maxSubstemLen = curr;
					if(minSubstemLen == 0 || curr < minSubstemLen)
						minSubstemLen = curr;
					curr = 0;
				}	
			}
			else if(path.charAt(i) == '6'){
				avgSBulgeLen++;
				curr++;
				if(path.charAt(i-1) == '5')
					sBulges++;
				if(i==length-1 || path.charAt(i+1) == '5'){
					if(curr > maxSBulgeLen)
						maxSBulgeLen = curr;
					if(minSBulgeLen == 0 || curr < minSBulgeLen)
						minSBulgeLen = curr;
					curr = 0;
				}
			}
		}
		if(aBulges != 0)
			avgABulgeLen /= aBulges;
		if(sBulges != 0)
			avgSBulgeLen /= sBulges;
		if(substems != 0)
			avgSubstemLen /= substems;
	}

	/* Prints out all feature values. */
	public void display()
	{
		System.out.println("Sequence length: "+length);
		System.out.println("Likelihood: "+prob);
		System.out.println("Proportion of adenine: "+comp[0]);
		System.out.println("Proportion of cytosine: "+comp[1]);
		System.out.println("Proportion of guanine: "+comp[2]);
		System.out.println("Proportion of thymine/uracil: "+comp[3]);
		System.out.println("Loop length: "+loop);
		System.out.println("Symmetric bulges: "+sBulges);
		System.out.println("Asymmetric bulges: "+aBulges);
		System.out.println("Substems: "+substems);
		System.out.println("Average symmetric bulge length: "+avgSBulgeLen);
		System.out.println("Average asymmetric bulge length: "+avgABulgeLen);
		System.out.println("Average substem length: "+avgSubstemLen);
		System.out.println("Maximum symmetric bulge length: "+maxSBulgeLen);
		System.out.println("Maximum asymmetric bulge length: "+maxABulgeLen);
		System.out.println("Maximum substem length: "+maxSubstemLen);
		System.out.println("Minimum symmetric bulge length: "+minSBulgeLen);
		System.out.println("Minimum asymmetric bulge length: "+minABulgeLen);
		System.out.println("Minimum substem length: "+minSubstemLen);
	}

	/* Returns an array of all feature values.
         *
         * @return An array containing all feature values
         */
	public double[] getValues()
	{
		double[] f = new double[18];
		f[0] = length;
		f[1] = prob;
		for(int i=0; i<3; i++)
			f[i+2] = comp[i];
		f[5] = loop;
		f[6] = sBulges;
		f[7] = aBulges;
		f[8] = substems;
		f[9] = avgSBulgeLen;
		f[10] = avgABulgeLen;
		f[11] = avgSubstemLen;
		f[12] = maxSBulgeLen;
		f[13] = maxABulgeLen;
		f[14] = maxSubstemLen;
		f[15] = minSBulgeLen;
		f[16] = minABulgeLen;
		f[17] = minSubstemLen;
		return f;
	}
}
