import java.io.*;

/** This class does the job of learning the parameters for the context-sensitive HMM representing  
 *  miRNA precursors, using labeled data in the form of the secondary miRNA-precursor structures. 
 *
 * @author Sumeet Agarwal
 * @version 15.7.2005.
 */     
public class Learn
{
    /* The main function; reads a set of miRNA secondary structures stored in particular format
     * (as in the file miRNA_hs.str), and uses them as labeled data to learn the csHMM     
     * parameters. The structure of the model is fixed, and coded into this program. 
     */
    public static void main(String args[])
    {
        int trans[][] = new int[5][8];
        int emit[][] = new int[5][4];
        int ctrans[][][] = new int[2][5][8];
        int cemit[][][] = new int[2][4][4];
        int num=0;
        try{
            InputStreamReader reader = new InputStreamReader(System.in);
            BufferedReader buf = new BufferedReader(reader);
            while(true)
            {           
                String header = buf.readLine();
                if(header == null)
                    break;
                String blank = buf.readLine();
                String lines[] = new String[5];
                for(int i=0; i<5; i++)
                    lines[i] = buf.readLine();
                blank = buf.readLine();         
                int len = lines[1].length();
                if(len < lines[2].length())
                    len = lines[2].length();
                int i=len-1;
                while(lines[2].charAt(i) != '|'){
                    for(int j=0; j<5; j++){
                        int c = toNum(lines[j], i);
                        if(c>=0){
                            emit[3][c]++;
                            trans[3][3]++;
                        }
                    }
                    i--;
                }
                trans[3][3]--;
                int prev = 3;
                int next = 3;   
                while(i>=0){
                    if(lines[2].charAt(i) == '|'){
                        int c1 = toNum(lines[1], i);
                        int c2 = toNum(lines[3], i);
                        emit[1][c1]++;
                        cemit[0][c1][c2]++;
                        trans[1][next]++;
                        if(prev < 5)        
                            trans[prev][5]++;
                        else
                            ctrans[prev%5][c1][5]++;
                        prev = 5;
                        next = 1;
                    }
                    else{
                        int c1 = toNum(lines[0], i);    
                        int c2 = toNum(lines[4], i);
                        if(c1 < 0){
                            emit[4][c2]++;
                            if(prev < 5)        
                                trans[prev][4]++;
                            else{
                                int j = i-1;
                                while(j>=0){
                                    if(toNum(lines[0], j) >= 0)
                                        break;
                                    j--;
                                }
                                if(j>=0)
                                    ctrans[prev%5][toNum(lines[0], j)][4]++;
                                else
                                    ctrans[prev%5][4][4]++;
                            }
                            prev = 4;
                        }
                        else if(c2 < 0){
                            emit[2][c1]++;
                            trans[2][next]++;
                            next = 2;
                        }
                        else{
                            emit[1][c1]++;
                            cemit[1][c1][c2]++;
                            trans[1][next]++;
                            if(prev < 5)        
                                trans[prev][6]++;
                            else
                                ctrans[prev%5][c1][6]++;
                            prev = 6;
                            next = 1;
                        }
                    }
                    i--;
                }
                if(prev < 5)        
                    trans[prev][7]++;
                else
                    ctrans[prev%5][4][7]++;
                trans[0][next]++;
                num++;
                //System.out.println(num);
            }
            //System.out.println("Learnt from "+num+" sequences");
            int tsum[] = new int[5];
            int esum[] = new int[5];
            int ctsum[][] = new int[2][5];
            int cesum[][] = new int[2][4];
            for(int n=0; n<5; n++){
                for(int m=0; m<8; m++){
                    tsum[n] += trans[n][m];
                    if(m<4)
                        esum[n] += emit[n][m];
                }
            }
            PrintWriter pw = new PrintWriter(System.out);
            for(int n=0; n<5; n++){
                for(int m=0; m<8; m++)
                    pw.print((float)trans[n][m]/tsum[n]+"\t");
                pw.println();   
            }
            pw.println();
            for(int n=0; n<5; n++){
                for(int m=0; m<4; m++){
                    if(esum[n]>0)
                        pw.print((float)emit[n][m]/esum[n]+"\t");
                    else
                        pw.print("0.0\t");
                }
                pw.println();   
            }
            pw.println();
            for(int l=0; l<2; l++){
                for(int n=0; n<5; n++){
                    for(int m=0; m<8; m++){
                        ctsum[l][n] += ctrans[l][n][m];
                        if(n<4 && m<4)
                            cesum[l][n] += cemit[l][n][m];
                    }
                }
            }
            for(int l=0; l<2; l++){
                for(int n=0; n<5; n++){
                    for(int m=0; m<8; m++)
                        pw.print((float)ctrans[l][n][m]/ctsum[l][n]+"\t");
                    pw.println();
                }
                pw.println();
            }
            for(int l=0; l<2; l++){
                for(int n=0; n<4; n++){
                    for(int m=0; m<4; m++)
                        pw.print((float)cemit[l][n][m]/cesum[l][n]+"\t");
                    pw.println();
                }
                pw.println();
            }
            pw.close();
        }
        catch(Exception e){
            System.out.println("Exception: "+e);
        }
    }
    
    /* Converts the bases in the input sequence to corresponding numbers.
         *
         * @param in The input string
         * @param i The position in the input string to be converted to a number
         *
         * @return The integer value corresponding to the base at the given position
         */ 
    static int toNum(String in, int i){
                int xi = -1;
        if(i < in.length()){
            char ci = in.charAt(i);
                switch(ci){
                    case 'a':
            case 'A':
                        xi = 0;
                        break;
                    case 'c':
            case 'C':
                        xi = 1;
                        break;
                    case 'g':
            case 'G':
                        xi = 2;
                        break;
                    case 'u':
            case 'U':
                        xi = 3;
                        break;
                }
        }
            return xi;
        }

}   
        
        
