import java.io.*;
import java.util.StringTokenizer;

/** This class extracts the likelihood feature for a set of input RNA sequences,
 * and stores the values in ARFF format. It uses the context-sensitive HMM as set out in Align.java to
 * generate the secondary structure and compute the likelihood.
 *
 * @author Sumeet Agarwal
 * @version 3.3.2010.
 */
public class Likelihood
{
	static InputStreamReader reader = new InputStreamReader(System.in);
	static BufferedReader console = new BufferedReader(reader);
		
	/* The main function; reads in sequence data from specified files; then generates the secondary 
 	 * structure feature values in the ARFF format and writes to stdout.
 	 *
 	 * Arguments: CSHMM parameters file, negative instances file, positive instances file.
         */
	public static void main(String args[])
	{
		String file;
		try{
			PrintWriter pw = new PrintWriter(System.out);
			pw.println("@relation mirna\n");
			pw.println("@attribute likelihood real");
			pw.println("@attribute class {0, 1}");		
			pw.println("\n@data");
			Align a = new Align();
			try{
				a.getParams(args[0]);
			}
			catch(Exception e){
				System.out.print("Please enter parameter file for csHMM: ");	
				a.getParams(console.readLine());
			}
			for(int clv=0; clv<=1; clv++){
		        	file = args[clv+1];
				FileInputStream str = new FileInputStream(file);
				InputStreamReader in = new InputStreamReader(str);
				BufferedReader buf = new BufferedReader(in);
				int i=0;
				String seq = buf.readLine();
				int exc = 0;
				while(!(seq == null)){
					try{
						String[] val = a.genPath(seq);
						double prob = Double.parseDouble(val[0]);
						String path = val[1];
						FV f = a.getFeatures(seq, prob, path);
						double[] vals = f.getValues();
						for(int j=1; j<2; j++)
							pw.print(vals[j]+",");
						pw.println(clv);
						i++;
					}
					catch(Exception e){
						if(seq.charAt(0) != '>'){
							System.err.println("Failed to align sequence (you may need to change the MAX_LEN parameter): "+seq);
							e.printStackTrace();
							exc++;
						}
					}
					seq = buf.readLine();
				}
				//System.err.println(i+" sequences read successfully");
			}	
			pw.close();
		}
		catch(Exception e){
			System.out.println("Exception: "+e);
		}
	}
}
