import java.io.*;
import weka.classifiers.Classifier;
import weka.classifiers.trees.J48;
//import weka.classifiers.functions.SMO;
import weka.core.*;

/**
 * Does the job of checking a given set of sequences for potential pre-miRNA candidates.
 * Uses the classifier(s) trained via a csHMM model to identify possible positive sequences.
 *
 * @author Sumeet Agarwal
 * @version 6.11.2010.
 */

public class RunTest
{

	/*
	 * Arguments are CSHMM parameter file, training instances file and an optional third argument which can be used to specify that predicted secondary structures should
	 * also be output along with the sequences (use -str for this). Sequences are read from stdin, output written to stdout.
	 */
	public static void main(String args[]) throws Exception
	{
		FileReader fr = new FileReader(args[1]);
		//Instances tdata = new Instances(fr);
		//fr = new FileReader(args[2]);
		Instances tdataL = new Instances(fr);
		//tdata.setClassIndex(tdata.numAttributes() - 1);
		tdataL.setClassIndex(tdataL.numAttributes() - 1);
		final int numc = 1;
		Classifier[] c = new Classifier[numc];
		c[0] = new J48();
		//c[1] = new J48();
		//c[2] = new SMO();
		//c[3] = new SMO();
		c[0].buildClassifier(tdataL);
		//c[1].buildClassifier(tdata);
		//c[2].buildClassifier(tdataL);
		//c[3].buildClassifier(tdata);
		Align2 a = new Align2();
		a.getParams(args[0]);
		InputStreamReader r = new InputStreamReader(System.in);
		BufferedReader console = new BufferedReader(r);
		//Instances data = new Instances(tdata, 1000000);
		//data.setClassIndex(data.numAttributes() - 1);
		Instances dataL = new Instances(tdataL, 1000000);
		dataL.setClassIndex(dataL.numAttributes() - 1);
		String line = console.readLine();
		int[] num = new int[numc];
		PrintWriter[] pw = new PrintWriter[numc];
		pw[0] = new PrintWriter(System.out);
		//pw[1] = new PrintWriter(args[3]+".predj48");
		//pw[2] = new PrintWriter(args[3]+".predsmo_l");
		//pw[3] = new PrintWriter(args[3]+".predsmo");
		while(line != null){
					try{
						String[] s = a.genPath(line);
						double prob = Double.parseDouble(s[0]);
						FV fv = a.getFeatures(line, prob, s[1]);
						double[] score = {fv.prob, 0};
						//Instance inst1 = new Instance(1.0, fv.getValues());
						//inst1.setDataset(data);
						//data.add(inst1);
						Instance inst2 = new Instance(1.0, score);
						inst2.setDataset(dataL);
						dataL.add(inst2);
						for(int k=0; k<numc; k++){
							Instance inst = inst2;
							//if(k%2==0)
							//inst = inst2;
							double[] distr = c[k].distributionForInstance(inst);
							if(distr[0]<distr[1]){
								num[k]++;
								pw[k].println(line+" "+score[0]);
								if(args.length==3 && args[2].equals("-str")){
									// Also generate and output predicted secondary structure
									String[] l = a.genFigure(line, s[1]);
									for(int i=0; i<5; i++)

										pw[k].println(l[i]);

									pw[k].println(); 

								}
							}
						}
					}
					catch(Exception e){
						//e.printStackTrace();
						//System.out.println(cand+"\n"+prob+"\n"+s[1]);
						//System.exit(1);
						//System.err.println("Could not fold; skipping: "+cand);
					}
				
			
			for(int k=0; k<numc; k++)
				pw[k].flush();
			line = console.readLine();
		}
		for(int k=0; k<numc; k++)
			pw[k].close();
	}
}
