import java.io.*;
import weka.classifiers.Classifier;
import weka.classifiers.trees.J48;
//import weka.classifiers.functions.SMO;
import weka.core.*;

/**
 * Does the job of scanning a given genome or set of genomic regions for potential pre-miRNA candidates.
 * Uses the classifier(s) trained via a csHMM model to identify possible positive sequences.
 *
 * @author Sumeet Agarwal
 * @version 5.8.2011
 */

public class ScanTest
{
	static int SHIFT=10; // Start of scanning window is shifted forward by this many positions every iteration
	static int MIN_SIZE=60; // Smallest length of sequence to be considered for identification as pre-miRNA
	static int MAX_SIZE=120; // Largest length to be considered

	/*
	 * Arguments are CSHMM parameter file, training instances file and offset (optional, to change the likelihood threshold and make the classifier more or less stringent). Genomic regions are read from stdin, output written to stdout.
	 */
	public static void main(String args[]) throws Exception
	{
		// This is the offset to be added to the likelihood threshold determined by the classifier (suggested range is within +/- 0.05)
		double OFFSET=0;
		try{
			OFFSET = Double.parseDouble(args[2]);
		}
		catch(Exception e){
			;
		}

		FileReader fr = new FileReader(args[1]);
		//Instances tdata = new Instances(fr);
		//fr = new FileReader(args[2]);
		Instances tdataL = new Instances(fr);
		//tdata.setClassIndex(tdata.numAttributes() - 1);
		tdataL.setClassIndex(tdataL.numAttributes() - 1);
		final int numc = 1;
		Classifier[] c = new Classifier[numc];
		c[0] = new J48();
		//c[1] = new J48();
		//c[2] = new SMO();
		//c[3] = new SMO();
		c[0].buildClassifier(tdataL);
		//c[1].buildClassifier(tdata);
		//c[2].buildClassifier(tdataL);
		//c[3].buildClassifier(tdata);
		Align2 a = new Align2();
		a.getParams(args[0]);
		InputStreamReader r = new InputStreamReader(System.in);
		BufferedReader console = new BufferedReader(r);
		//Instances data = new Instances(tdata, 1000000);
		//data.setClassIndex(data.numAttributes() - 1);
		Instances dataL = new Instances(tdataL, 1000000);
		dataL.setClassIndex(dataL.numAttributes() - 1);
		String line = "";
		int[] num = new int[numc];
		PrintWriter[] pw = new PrintWriter[numc];
		pw[0] = new PrintWriter(System.out);
		//pw[1] = new PrintWriter(args[3]+".predj48");
		//pw[2] = new PrintWriter(args[3]+".predsmo_l");
		//pw[3] = new PrintWriter(args[3]+".predsmo");
		while(line != null){
			int ch = 0;
			String region="";
			while(true){
				ch = console.read();
				if(ch == '\n')
					continue;
				if(ch == '>'){
					//for(int k=0; k<numc; k++)
					//	System.err.println(num[k] + " candidates detected by classifier "+(k+1)+" so far");
					region = (char)ch+console.readLine();
					break;
				}
				if(ch < 'A')
					break;
				line += (char)ch;
			}
			//System.out.println(line);
			int[] start = new int[numc];
			int[] end = new int[numc];
			double[] max = new double[numc];
			String[] best = new String[numc];
			String[] str = new String[numc];
			for(int i=0; i<=line.length()-MIN_SIZE; i+=SHIFT){
				for(int j=i+MIN_SIZE; j<=i+MAX_SIZE; j++){
					if(j > line.length())
						break;
					String cand = line.substring(i,j);
					//System.out.println(cand);
					try{
						String[] s = a.genPath(cand);
						double prob = Double.parseDouble(s[0]);
						FV fv = a.getFeatures(cand, prob, s[1]);
						double[] score = {fv.prob-OFFSET, 0};
						//Instance inst1 = new Instance(1.0, fv.getValues());
						//inst1.setDataset(data);
						//data.add(inst1);
						Instance inst2 = new Instance(1.0, score);
						inst2.setDataset(dataL);
						dataL.add(inst2);
						for(int k=0; k<numc; k++){
							Instance inst = inst2;
							//if(k%2==0)
							//inst = inst2;
							double[] distr = c[k].distributionForInstance(inst);
							if(distr[0]<distr[1]){
								if(i>=end[k] && max[k]>0){
									num[k]++;
									pw[k].println("Start: "+start[k]+" End: "+end[k]);
									pw[k].println(best[k]+" "+(max[k]+OFFSET));
									// Also generate and output predicted secondary structure
									String[] l = a.genFigure(best[k], str[k]);
									for(int m=0; m<5; m++)

										pw[k].println(l[m]);

									pw[k].println(); 
								}
								if(i<end[k] && score[0] <= max[k]);
								else{
									start[k] = i;
									end[k] = j;
									max[k] = score[0];
									best[k] = cand;
									str[k] = s[1];
								}
							}
						}
					}
					catch(Exception e){
						//System.err.println("Could not fold; skipping: "+cand);
						//e.printStackTrace();
						//System.out.println(cand+"\n"+prob+"\n"+s[1]);
						//System.exit(1);
						
					}
				}
			}
			for(int k=0; k<numc; k++){
				if(max[k] > 0){
					num[k]++;
					pw[k].println("Start: "+start[k]+" End: "+end[k]);
					pw[k].println(best[k]+" "+(max[k]+OFFSET));
					// Also generate and output predicted secondary structure
					String[] l = a.genFigure(best[k], str[k]);
					for(int i=0; i<5; i++)
						pw[k].println(l[i]);
					pw[k].println();
				}
				pw[k].println(region);
				pw[k].flush();
			}
			if(ch<0)
				break;
			line = "";
		}
		for(int k=0; k<numc; k++)
			pw[k].close();
	}
}
